def portfolio_margin_account(self, **kwargs):
    """Get Portfolio Margin Account Info (USER_DATA)

    GET /sapi/v1/portfolio/account

    https://binance-docs.github.io/apidocs/spot/en/#get-portfolio-margin-account-info-user_data

    Keyword Args:
      recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/portfolio/account", {**kwargs})



from binance.lib.utils import check_required_parameters


def convert_trade_history(self, startTime: int, endTime: int, **kwargs):
    """Convert Trade History (USER_DATA)

    Get convert history for a specific account.

    GET /sapi/v1/convert/tradeFlow

    https://binance-docs.github.io/apidocs/spot/en/#convert-endpoints

    Args:
        startTime (int)
        endTime (int)
    Keyword Args:
        limit (int, optional): Default Value: 100; Max Value: 1000
        recvWindow (int, optional)
    """
    check_required_parameters([[startTime, "startTime"], [endTime, "endTime"]])

    url_path = "/sapi/v1/convert/tradeFlow"
    payload = {"startTime": startTime, "endTime": endTime, **kwargs}
    return self.sign_request("GET", url_path, payload)



from binance.lib.utils import check_required_parameter


def loan_history(self, asset: str, **kwargs):
    """Get Crypto Loans Income History (USER_DATA)

    GET /sapi/v1/loan/income

    https://binance-docs.github.io/apidocs/spot/en/#get-crypto-loans-income-history-user_data

    Args:
      asset (str)
    Keyword Args:
      type (str, optional): All types will be returned by default.
                            borrowIn, collateralSpent, repayAmount, collateralReturn (collateral return after repayment),
                            addCollateral, removeCollateral, collateralReturnAfterLiquidation
      startTime (int, optional)
      endTime (int, optional)
      limit (int, optional): default 20, max 100
      recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(asset, "asset")

    payload = {"asset": asset, **kwargs}
    return self.sign_request("GET", "/sapi/v1/loan/income", payload)



from binance.error import ParameterArgumentError
from binance.lib.utils import (
    check_required_parameter,
    check_type_parameter,
    convert_list_to_json_array,
)
from binance.lib.utils import check_required_parameters


def ping(self):
    """Test Connectivity
    Test connectivity to the Rest API.

    GET /api/v3/ping

    https://binance-docs.github.io/apidocs/spot/en/#test-connectivity

    """

    url_path = "/api/v3/ping"
    return self.query(url_path)


def time(self):
    """Check Server Time
    Test connectivity to the Rest API and get the current server time.

    GET /api/v3/time

    https://binance-docs.github.io/apidocs/spot/en/#check-server-time

    """

    url_path = "/api/v3/time"
    return self.query(url_path)


def exchange_info(self, symbol: str = None, symbols: list = None):
    """Exchange Information
    Current exchange trading rules and symbol information

    GET /api/v3/exchangeinfo

    https://binance-docs.github.io/apidocs/spot/en/#exchange-information

     Args:
        symbol (str, optional): the trading pair
        symbols (list, optional): list of trading pairs
    """

    url_path = "/api/v3/exchangeInfo"
    if symbol and symbols:
        raise ParameterArgumentError("symbol and symbols cannot be sent together.")
    check_type_parameter(symbols, "symbols", list)
    params = {"symbol": symbol, "symbols": convert_list_to_json_array(symbols)}
    return self.query(url_path, params)


def depth(self, symbol: str, **kwargs):
    """Get orderbook.

    GET /api/v3/depth

    https://binance-docs.github.io/apidocs/spot/en/#order-book

    Args:
        symbol (str): the trading pair
    Keyword Args:
        limit (int, optional): limit the results. Default 100; max 5000. If limit > 5000, then the response will truncate to 5000.
    """

    check_required_parameter(symbol, "symbol")
    params = {"symbol": symbol, **kwargs}
    return self.query("/api/v3/depth", params)


def trades(self, symbol: str, **kwargs):
    """Recent Trades List
    Get recent trades (up to last 500).

    GET /api/v3/trades

    https://binance-docs.github.io/apidocs/spot/en/#recent-trades-list

    Args:
        symbol (str): the trading pair
    Keyword Args:
        limit (int, optional): limit the results. Default 500; max 1000.
    """
    check_required_parameter(symbol, "symbol")
    params = {"symbol": symbol, **kwargs}
    return self.query("/api/v3/trades", params)


def historical_trades(self, symbol: str, **kwargs):
    """Old Trade Lookup
    Get older market trades.

    GET /api/v3/historicalTrades

    https://binance-docs.github.io/apidocs/spot/en/#old-trade-lookup

    Args:
        symbol (str): the trading pair
    Keyword Args:
        limit (int, optional): limit the results. Default 500; max 1000.
        formId (int, optional): trade id to fetch from. Default gets most recent trades.
    """
    check_required_parameter(symbol, "symbol")
    params = {"symbol": symbol, **kwargs}
    return self.limit_request("GET", "/api/v3/historicalTrades", params)


def agg_trades(self, symbol: str, **kwargs):
    """Compressed/Aggregate Trades List

    GET /api/v3/aggTrades

    https://binance-docs.github.io/apidocs/spot/en/#compressed-aggregate-trades-list

    Args:
        symbol (str): the trading pair
    Keyword Args:
        limit (int, optional): limit the results. Default 500; max 1000.
        formId (int, optional): id to get aggregate trades from INCLUSIVE.
        startTime (int, optional): Timestamp in ms to get aggregate trades from INCLUSIVE.
        endTime (int, optional): Timestamp in ms to get aggregate trades until INCLUSIVE.
    """

    check_required_parameter(symbol, "symbol")
    params = {"symbol": symbol, **kwargs}
    return self.query("/api/v3/aggTrades", params)


def klines(self, symbol: str, interval: str, **kwargs):
    """Kline/Candlestick Data

    GET /api/v3/klines

    https://binance-docs.github.io/apidocs/spot/en/#kline-candlestick-data

    Args:
        symbol (str): the trading pair
        interval (str): the interval of kline, e.g 1m, 5m, 1h, 1d, etc.
    Keyword Args:
        limit (int, optional): limit the results. Default 500; max 1000.
        startTime (int, optional): Timestamp in ms to get aggregate trades from INCLUSIVE.
        endTime (int, optional): Timestamp in ms to get aggregate trades until INCLUSIVE.
    """
    check_required_parameters([[symbol, "symbol"], [interval, "interval"]])

    params = {"symbol": symbol, "interval": interval, **kwargs}
    return self.query("/api/v3/klines", params)


def avg_price(self, symbol: str):
    """Current Average Price

    GET /api/v3/avgPrice

    https://binance-docs.github.io/apidocs/spot/en/#current-average-price

    Args:
        symbol (str): the trading pair
    """

    check_required_parameter(symbol, "symbol")
    params = {
        "symbol": symbol,
    }
    return self.query("/api/v3/avgPrice", params)


def ticker_24hr(self, symbol: str = None, symbols: list = None):
    """24hr Ticker Price Change Statistics

    GET /api/v3/ticker/24hr

    https://binance-docs.github.io/apidocs/spot/en/#24hr-ticker-price-change-statistics

    Args:
        symbol (str, optional): the trading pair
        symbols (list, optional): list of trading pairs
    """

    if symbol and symbols:
        raise ParameterArgumentError("symbol and symbols cannot be sent together.")
    check_type_parameter(symbols, "symbols", list)
    params = {"symbol": symbol, "symbols": convert_list_to_json_array(symbols)}
    return self.query("/api/v3/ticker/24hr", params)


def ticker_price(self, symbol: str = None, symbols: list = None):
    """Symbol Price Ticker

    GET /api/v3/ticker/price

    https://binance-docs.github.io/apidocs/spot/en/#symbol-price-ticker

    Args:
        symbol (str, optional): the trading pair
        symbols (list, optional): list of trading pairs
    """

    if symbol and symbols:
        raise ParameterArgumentError("symbol and symbols cannot be sent together.")
    check_type_parameter(symbols, "symbols", list)
    params = {"symbol": symbol, "symbols": convert_list_to_json_array(symbols)}
    return self.query("/api/v3/ticker/price", params)


def book_ticker(self, symbol: str = None, symbols: list = None):
    """Symbol Order Book Ticker

    GET /api/v3/ticker/bookTicker

    https://binance-docs.github.io/apidocs/spot/en/#symbol-order-book-ticker

    Args:
        symbol (str, optional): the trading pair
        symbols (list, optional): list of trading pairs
    """

    if symbol and symbols:
        raise ParameterArgumentError("symbol and symbols cannot be sent together.")
    check_type_parameter(symbols, "symbols", list)
    params = {"symbol": symbol, "symbols": convert_list_to_json_array(symbols)}
    return self.query("/api/v3/ticker/bookTicker", params)


def rolling_window_ticker(self, symbol: str = None, symbols: list = None, **kwargs):
    """Rolling window price change statistics

    The window used to compute statistics is typically slightly wider than requested windowSize.

    openTime for /api/v3/ticker always starts on a minute, while the closeTime is the current time of the request. As such, the effective window might be up to 1 minute wider than requested.

    E.g. If the closeTime is 1641287867099 (January 04, 2022 09:17:47:099 UTC) , and the windowSize is 1d. the openTime will be: 1641201420000 (January 3, 2022, 09:17:00 UTC)

    Weight(IP): 2 for each requested symbol regardless of windowSize.

    The weight for this request will cap at 100 once the number of symbols in the request is more than 50.

    GET /api/v3/ticker

    https://binance-docs.github.io/apidocs/spot/en/#rolling-window-price-change-statistics

    Args:
        symbol (str, optional): the trading pair
        symbols (str, optional): : list of trading pairs. The maximum number of symbols allowed in a request is 100.
    Keyword Args:
        windowSize (str, optional): Defaults to 1d if no parameter provided.
    """

    if symbol and symbols:
        raise ParameterArgumentError("symbol and symbols cannot be sent together.")
    check_type_parameter(symbols, "symbols", list)
    params = {
        "symbol": symbol,
        "symbols": convert_list_to_json_array(symbols),
        **kwargs,
    }
    url_path = "/api/v3/ticker"
    return self.query(url_path, params)



from binance.lib.utils import (
    check_required_parameter,
)
from binance.lib.utils import check_required_parameters


def staking_product_list(self, product: str, **kwargs):
    """Get Staking Product List (USER_DATA)

    Get available Staking product list.

    Weight(IP): 1

    GET /sapi/v1/staking/productList

    https://binance-docs.github.io/apidocs/spot/en/#get-staking-product-list-user_data

    Args:
        product (str)
    Keyword Args:
        asset (str, optional)
        current (int, optional): Currently querying page. Start from 1. Default:1
        size (int, optional): Default:10, Max:100
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameter(product, "product")

    params = {"product": product, **kwargs}
    url_path = "/sapi/v1/staking/productList"
    return self.sign_request("GET", url_path, params)


def staking_purchase_product(
    self, product: str, productId: str, amount: float, **kwargs
):
    """Purchase Staking Product (USER_DATA)

    Weight(IP): 1

    POST /sapi/v1/staking/purchase

    https://binance-docs.github.io/apidocs/spot/en/#purchase-staking-product-user_data

    Args:
        product (str)
        productId (str)
        amount (float)
    Keyword Args:
        renewable (str, optional): true or false, default false. Active if product is `STAKING` or `L_DEFI`
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameters(
        [[product, "product"], [productId, "productId"], [amount, "amount"]]
    )

    params = {"product": product, "productId": productId, "amount": amount, **kwargs}
    url_path = "/sapi/v1/staking/purchase"
    return self.sign_request("POST", url_path, params)


def staking_redeem_product(self, product: str, productId: str, **kwargs):
    """Redeem Staking Product (USER_DATA)

    Redeem Staking product. Locked staking and Locked DeFI staking belong to early redemption, redeeming in advance will result in loss of interest that you have earned.

    Weight(IP): 1

    POST /sapi/v1/staking/redeem

    https://binance-docs.github.io/apidocs/spot/en/#redeem-staking-product-user_data

    Args:
        product (str)
        productId (str)
    Keyword Args:
        positionId (str, optional): Mandatory if product is `STAKING` or `L_DEFI`
        amount (float, optional): Mandatory if product is `F_DEFI`
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameters([[product, "product"], [productId, "productId"]])

    params = {"product": product, "productId": productId, **kwargs}
    url_path = "/sapi/v1/staking/redeem"
    return self.sign_request("POST", url_path, params)


def staking_product_position(self, product: str, **kwargs):
    """Get Staking Product Position (USER_DATA)

    Weight(IP): 1

    GET /sapi/v1/staking/position

    https://binance-docs.github.io/apidocs/spot/en/#get-staking-product-position-user_data

    Args:
        product (str)
    Keyword Args:
        productId (str, optional)
        asset (str, optional)
        current (int, optional): Currently querying the page. Start from 1. Default:1
        size (int, optional): Default:10, Max:100
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameter(product, "product")

    params = {"product": product, **kwargs}
    url_path = "/sapi/v1/staking/position"
    return self.sign_request("GET", url_path, params)


def staking_history(self, product: str, txnType: str, **kwargs):
    """Get Staking History (USER_DATA)

    Weight(IP): 1

    GET /sapi/v1/staking/stakingRecord

    https://binance-docs.github.io/apidocs/spot/en/#get-staking-history-user_data

    Args:
        product (str)
        txnType (str)
    Keyword Args:
        asset (str, optional)
        startTime (int, optional)
        endTime (int, optional)
        current (int, optional): Currently querying the page. Start from 1. Default:1
        size (int, optional): Default:10, Max:100
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameters([[product, "product"], [txnType, "txnType"]])

    params = {"product": product, "txnType": txnType, **kwargs}
    url_path = "/sapi/v1/staking/stakingRecord"
    return self.sign_request("GET", url_path, params)


def staking_set_auto_staking(
    self, product: str, positionId: str, renewable: str, **kwargs
):
    """Set Auto Staking (USER_DATA)

    Set auto staking on Locked Staking or Locked DeFi Staking

    Weight(IP): 1

    POST /sapi/v1/staking/setAutoStaking

    https://binance-docs.github.io/apidocs/spot/en/#set-auto-staking-user_data

    Args:
        product (str)
        positionId (str)
        renewable (str)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameters(
        [[product, "product"], [positionId, "positionId"], [renewable, "renewable"]]
    )

    params = {
        "product": product,
        "positionId": positionId,
        "renewable": renewable,
        **kwargs,
    }
    url_path = "/sapi/v1/staking/setAutoStaking"
    return self.sign_request("POST", url_path, params)


def staking_product_quota(self, product: str, productId: str, **kwargs):
    """Get Personal Left Quota of Staking Product (USER_DATA)

    Weight(IP): 1

    GET /sapi/v1/staking/personalLeftQuota

    https://binance-docs.github.io/apidocs/spot/en/#get-personal-left-quota-of-staking-product-user_data

    Args:
        product (str)
        productId (str)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameters([[product, "product"], [productId, "productId"]])

    params = {"product": product, "productId": productId, **kwargs}
    url_path = "/sapi/v1/staking/personalLeftQuota"
    return self.sign_request("GET", url_path, params)
